# Copyright (C) 2014 ETH Zurich, Institute for Astronomy

"""
Tests for `hope` module.
"""
from __future__ import print_function, division, absolute_import, unicode_literals

import numpy as np
import hope, itertools, pytest, sys, sysconfig, os, shutil

from test.utilities import random, check, make_test, JENKINS, min_dtypes, dtypes, shapes, setup_module, setup_method, teardown_module

@pytest.mark.parametrize("dtype", dtypes)
def test_assignment(dtype):
    def fkt(a, b): a[:] = b
    (ao, ah), (b, _) = random(dtype, [10]), random(dtype, [10])
    fkt(ao, b), hope.jit(fkt)(ah, b)
    assert check(ao, ah)
    fkt(ao, b), hope.jit(fkt)(ah, b)
    assert check(ao, ah)

@pytest.mark.parametrize("dtype", dtypes)
def test_index_1d(dtype):
    def fkt(a, b): a[5] = b[3]
    (ao, ah), (b, _) = random(dtype, [10]), random(dtype, [10])
    fkt(ao, b), hope.jit(fkt)(ah, b)
    assert check(ao, ah)
    fkt(ao, b), hope.jit(fkt)(ah, b)
    assert check(ao, ah)

@pytest.mark.parametrize("dtype", dtypes)
def test_index_2d(dtype):
    def fkt(a, b): a[4, 2] = b[3, 4]
    (ao, ah), (b, _) = random(dtype, [5, 5]), random(dtype, [5, 5])
    fkt(ao, b), hope.jit(fkt)(ah, b)
    assert check(ao, ah)
    fkt(ao, b), hope.jit(fkt)(ah, b)
    assert check(ao, ah)

@pytest.mark.parametrize("dtype", dtypes)
def test_scalar(dtype):
    def fkt(a, b): a[3] = b
    (ao, ah), (b, _) = random(dtype, [10]), random(dtype, [])
    fkt(ao, b), hope.jit(fkt)(ah, b)
    assert check(ao, ah)

@pytest.mark.parametrize("dtype", dtypes)
def test_index_1d(dtype):
    def fkt(a, b): a[5] = b[3]
    (ao, ah), (b, _) = random(dtype, [10]), random(dtype, [10])
    fkt(ao, b), hope.jit(fkt)(ah, b)
    assert check(ao, ah)
    fkt(ao, b), hope.jit(fkt)(ah, b)
    assert check(ao, ah)

@pytest.mark.parametrize("dtype", dtypes)
def test_slice_1d_1(dtype):
    def fkt(a, b): a[2:5] = b[6:9]
    (ao, ah), (b, _) = random(dtype, [10]), random(dtype, [10])
    fkt(ao, b), hope.jit(fkt)(ah, b)
    assert check(ao, ah)
    fkt(ao, b), hope.jit(fkt)(ah, b)
    assert check(ao, ah)

@pytest.mark.parametrize("dtype", dtypes)
def test_slice_1d_2(dtype):
    def test(a, b): a[:5] = b[3:8]
    (ao, ah), (b, _) = random(dtype, [10]), random(dtype, [10])
    test(ao, b), hope.jit(test)(ah, b)
    assert check(ao, ah)
    test(ao, b), hope.jit(test)(ah, b)
    assert check(ao, ah)

@pytest.mark.parametrize("dtype", dtypes)
def test_slice_1d_3(dtype):
    def fkt(a, b): a[2:] = b[1:9]
    (ao, ah), (b, _) = random(dtype, [10]), random(dtype, [10])
    fkt(ao, b), hope.jit(fkt)(ah, b)
    assert check(ao, ah)
    fkt(ao, b), hope.jit(fkt)(ah, b)
    assert check(ao, ah)

@pytest.mark.parametrize("dtype,shape", itertools.product(dtypes, shapes[1:]))
@make_test
def test_assign(a, b, c): c[:] = a



# Copyright (C) 2014 ETH Zurich, Institute for Astronomy

"""
Test functions for `hope` module.
"""
from __future__ import print_function, division, absolute_import, unicode_literals

import hope
import numpy as np

from test.utilities import check


class SubCls(object):
    def __init__(self):
        self.i = 2


class Cls(object):
    def __init__(self):
        self.i = 1
        self.f = 1.
        self.ai = np.array([1, 2, 3], dtype=np.uint32)
        self.af = np.array([1, 2, 3], dtype=np.float32)
        self.res = np.array([0, 0, 0], dtype=np.float32)
        self.obj = SubCls()

    def fkt_cb(self):
        return self.i

    # @hope.jit
    # def fkt_jit(self):
    #   return self.f

    @hope.jit
    def fkt_1(self):
        self.res[:] = self.af

    @hope.jit
    def fkt_2(self):
        return self.fkt_cb()

    # @hope.jit
    # def fkt_3(self):
    #     return self.fkt_jit()

    @hope.jit
    def fkt_4(self):
        return self.obj.i


def test_cls_1():
    inst = Cls()
    inst.fkt_1()
    assert check(inst.af, inst.res)
    inst.af = np.array([1, 2, 3], dtype=np.int_)
    inst.fkt_1()
    assert check(inst.af, inst.res)


def test_cls_2():
    inst = Cls()
    assert check(inst.fkt_2(), inst.i)

# def test_cls_3():
#   inst = Cls()
#   assert check(inst.fkt_3(), inst.f)


def test_cls_4():
    inst = Cls()
    assert check(inst.fkt_4(), inst.obj.i)


if __name__ == '__main__':
    test_cls_1()
    test_cls_2()
    # test_cls_3()
    test_cls_4()



# Copyright (c) 2013 ETH Zurich, Institute of Astronomy, Lukas Gamper <lukas.gamper@usystems.ch>

from __future__ import print_function, division, absolute_import, unicode_literals


from hope.exceptions import UnsupportedCompilerException

CXX_FLAGS = {
    "clang": ["-Wall", "-Wno-unused-variable", "-march=native", "-stdlib=libc++", "-std=c++11"],
    "icc": ["-Wall", "-Wno-unused-variable", "-march=native", "-stdlib=libc++", "-std=c++11"],
    "gcc-mac": ["-Wall", "-Wno-unused-variable", "-std=c++11", "-msse4.2"],
    "gcc-linux": ["-Wall", "-Wno-unused-variable", "-std=c++11"]
}

DARWIN_KEY = "Darwin"
LINUX_KEY = "Linux"

#TODO: add check for other compilers
MIN_GCC_VERSION = "4.7.0"
SUPPORTED_VERSIONS = {"gcc": MIN_GCC_VERSION,
                      "gcc-linux": MIN_GCC_VERSION
                      }

# Enable faster but unsecure Math
def enableUnsaveMath():
    """
    Enable the fast-math and associative-math flags in the c++ compiler.

    .. warning::
        using these flags, ``nan`` and ``inf`` are not propageted properly.
        Only use these flags if you know what you are doing!
    """
    config.cxxflags += ["-fassociative-math", "-ffast-math"]


# Disable faster but unsecure Math
def disableUnsaveMath():
    """
    Disable the fast-math and associative-math flags in the c++ compiler.
    """
    del config.cxxflags[config.cxxflags.index("-ffast-math")]
    del config.cxxflags[config.cxxflags.index("-fassociative-math")]


# HOPE requires the compiler to compile with c++11 features enabled
def get_cxxflags():
    from distutils.ccompiler import new_compiler
    from distutils.sysconfig import customize_compiler
    from distutils import sysconfig
    from platform import system
    if system() == DARWIN_KEY:
        CXX_FLAGS["gcc"] = CXX_FLAGS["gcc-mac"]
        CXX_FLAGS["cc"] = CXX_FLAGS["clang"]
        CXX_FLAGS["c++"] = CXX_FLAGS["clang"]
    elif system() == LINUX_KEY:
        CXX_FLAGS["gcc"] = CXX_FLAGS["gcc-linux"]
        CXX_FLAGS["cc"] = CXX_FLAGS["gcc"]
        CXX_FLAGS["c++"] = CXX_FLAGS["gcc"]
    else:
        raise UnsupportedCompilerException("System: %s is not supported by HOPE" % system())
    
    sysconfig.get_config_vars() #init vars
    compiler = new_compiler()
    customize_compiler(compiler)
    compiler_name = compiler.compiler[0].split("/")[-1]
    
    _check_version(compiler_name, compiler.compiler[0])
    
    for name, flags in CXX_FLAGS.items():
        if compiler_name.startswith(name):
            return flags
    raise UnsupportedCompilerException("Unknown compiler: {0}".format(compiler_name))

def _check_version(compiler_name, compiler_exec):
    if compiler_name in SUPPORTED_VERSIONS.keys():
        import commands
        from distutils.version import StrictVersion

        version = commands.getoutput(compiler_exec + ' -dumpversion')
        if StrictVersion(version) < StrictVersion(SUPPORTED_VERSIONS[compiler_name]):
            raise UnsupportedCompilerException("Compiler '%s' with version '%s' is not supported. Minimum version is '%s'"%(compiler_name, 
                                                                                                                            version, 
                                                                                                                            SUPPORTED_VERSIONS[compiler_name]))
        print(version)



# Copyright (C) 2014 ETH Zurich, Institute for Astronomy

"""
Test operators for `hope` module.
"""
from __future__ import print_function, division, absolute_import, unicode_literals

import numpy as np
import hope, itertools, pytest, sys, sysconfig, os, shutil

from test.utilities import random, check, make_test, JENKINS, min_dtypes, dtypes, shapes, setup_module, setup_method, teardown_module

@pytest.mark.parametrize("dtype,shape", itertools.product([dtype for dtype in dtypes if issubclass(dtype, np.integer) or dtype == int], shapes))
@make_test
def test_binary_or(a, b, c): return a | b

@pytest.mark.parametrize("dtype,shape", itertools.product([dtype for dtype in dtypes if issubclass(dtype, np.integer) or dtype == int], shapes))
@make_test
def test_binary_xor(a, b, c): return a ^ b

@pytest.mark.parametrize("dtype,shape", itertools.product([dtype for dtype in dtypes if issubclass(dtype, np.integer) or dtype == int], shapes))
@make_test
def test_binary_and(a, b, c): return a & b

@pytest.mark.parametrize("dtype,shape", itertools.product([dtype for dtype in dtypes if issubclass(dtype, np.integer) or dtype == int], shapes[1:]))
@make_test
def test_augmented_or(a, b, c): c[:] |= a

@pytest.mark.parametrize("dtype,shape", itertools.product([dtype for dtype in dtypes if issubclass(dtype, np.integer) or dtype == int], shapes[1:]))
@make_test
def test_augmented_xor(a, b, c): c[:] ^= a

@pytest.mark.parametrize("dtype,shape", itertools.product([dtype for dtype in dtypes if issubclass(dtype, np.integer) or dtype == int], shapes[1:]))
@make_test
def test_augmented_and(a, b, c): c[:] &= a



# Copyright (c) 2013 ETH Zurich, Institute of Astronomy, Lukas Gamper <lukas.gamper@usystems.ch>
from __future__ import print_function, division, absolute_import, unicode_literals


__all__ = ["jit", "config", "serialize", "unserialize"]
__author__ = "Lukas Gamper"
__email__ = "hope@phys.ethz.ch"
__version__ = "0.3.0"
__credits__ = "ETH Zurich, Institute for Astronomy"

from hope import config
from hope.jit import jit
from hope.serialization import serialize, unserialize
from hope.options import enableUnsaveMath, disableUnsaveMath

from hope.exp import exp

# save hope version in compiled files
config.version = __version__
